#include <stdio.h>
#include "SharedTypes.h"
#include "gramatica.h"
#include "lexico.h"

FILE* g_codeOut;

extern CSList<SYMBOL>   symTable;
extern CSList<LABEL>    labels;
extern CSList<CUARTETO> cuartetos;
extern CSList<char*>  stringList;

extern void ErrorMessage(char* szMsg);

/////////////////////////////////////////////////////////////////////
// Funciones de ayuda para la generacin de cdigo

inline void Code(char* format, char* param)
{
	fputc('\t', g_codeOut);
	fprintf(g_codeOut, format, param);
	fputc('\n', g_codeOut);
}

inline void Code(char* format, int param)
{
	fputc('\t', g_codeOut);
	fprintf(g_codeOut, format, param);
	fputc('\n', g_codeOut);
}

inline void Code(char* format, char* param1, int param2)
{
	fputc('\t', g_codeOut);
	fprintf(g_codeOut, format, param1, param2);
	fputc('\n', g_codeOut);
}

inline void Code(char* format, int param1, char* param2)
{
	fputc('\t', g_codeOut);
	fprintf(g_codeOut, format, param1, param2);
	fputc('\n', g_codeOut);
}

inline void Code(char* format)
{
	fputc('\t', g_codeOut);
	fprintf(g_codeOut, format);
	fputc('\n', g_codeOut);
}

inline char* GetName1(CUARTETO& cuarteto)
{
	return symTable.GetAt(cuarteto.arg1.pSymbol).name;
}

inline char* GetName2(CUARTETO& cuarteto)
{
	return symTable.GetAt(cuarteto.arg2.pSymbol).name;
}

inline char* GetNameR(CUARTETO& cuarteto)
{
	return symTable.GetAt(cuarteto.res.pSymbol).name;
}

inline char* GetLabel1(CUARTETO& cuarteto)
{
	return labels.GetAt(cuarteto.arg1.pLabel).name;
}

inline char* GetLabelR(CUARTETO& cuarteto)
{
	return labels.GetAt(cuarteto.res.pLabel).name;
}

inline void MovAxArg1(CUARTETO& c)
{
	if (c.disc & SYM1)
		Code("mov ax, DS:[%s]", GetName1(c));
	else
		Code("mov ax, %dd", c.arg1.nConst);
}

inline void MovResAx(CUARTETO& c)
{
	Code("mov DS:[%s], ax", GetNameR(c));
}

inline void OpArg1(char* op, CUARTETO& c)
{
	if (c.disc & SYM1)
		fprintf(g_codeOut, "\t%s DS:[%s]\n", op, GetName1(c));
	else
		fprintf(g_codeOut, "\t%s %dd\n", op, c.arg1.nConst);
}

inline void OpArg2(char* op, CUARTETO& c)
{
	if (c.disc & SYM2)
		fprintf(g_codeOut, "\t%s DS:[%s]\n", op, GetName2(c));
	else
		fprintf(g_codeOut, "\t%s %dd\n", op, c.arg2.nConst);
}


/////////////////////////////////////////////////////////////////////
// Funcin de generacin de cdigo

void CodeWriteInt();
void CodeWriteString();
void CodeReadInt();
void CodeNewLine();

void GenerarCodigo(char* path)
{
	NODEPOS pos;
	bool bCodeWriteInt = false, bCodeWriteString = false,
		 bCodeReadInt = false, bCodeNewLine = false;
	
	g_codeOut = fopen(path, "wt" );

	if (g_codeOut == NULL) {
		ErrorMessage("Error al abrir el archivo .asm");
		return;
	}
	
	Code(".MODEL small");
	Code(".STACK 100h");
	Code(".DATA");
	
	/* Agregar la tabla de smbolos */

	pos = symTable.GetHeadPos();

	while (pos)
		Code("%s\tDW\t0", symTable.GetNext(pos).name);

	pos = stringList.GetHeadPos();
	int strSubfix;

	fputc('\n', g_codeOut);

	while (pos) {
		strSubfix = (int)pos;
		Code("str%d\tDB\t'%s$'", strSubfix, stringList.GetNext(pos));
	}

	fputc('\n', g_codeOut);	// Una nueva lnea para legibilidad
	
	/* Traducir los cuartetos a assembler */
	Code(".CODE");
	Code("mov ax, @data");
	Code("mov ds, ax");		// DS debe apuntar al segmento de datos

	pos = cuartetos.GetHeadPos();
	
	while (pos) {
		CUARTETO& c = cuartetos.GetNext(pos);
		
		switch (c.label) {
			case '+':
				MovAxArg1(c);
				OpArg2("add ax,", c);
				MovResAx(c);
				break;
			case '-':
				MovAxArg1(c);
				OpArg2("sub ax,", c);
				MovResAx(c);
				break;
			case '*':
				MovAxArg1(c);
				if (c.disc & SYM2)
					Code("imul DS:[%s]", GetName2(c));
				else {
					Code("mov bx, %dd", c.arg2.nConst);
					Code("imul bx");
				}
				MovResAx(c);
				break;
			case '/':
				MovAxArg1(c);
				Code("cwd");
				if (c.disc & SYM2)
					Code("idiv DS:[%s]", GetName2(c));
				else {
					Code("mov bx, %dd", c.arg2.nConst);
					Code("idiv bx");
				}
				MovResAx(c);
				break;
			case '>':
				OpArg1("mov cx,", c);
				OpArg2("cmp cx,", c);
				Code("jg %s", GetLabelR(c));
				break;
			case '<':
				OpArg1("mov cx,", c);
				OpArg2("cmp cx,", c);
				Code("jl %s", GetLabelR(c));
				break;
			case MAYIG:
				OpArg1("mov cx,", c);
				OpArg2("cmp cx,", c);
				Code("jge %s", GetLabelR(c));
				break;
			case MENIG:
				OpArg1("mov cx,", c);
				OpArg2("cmp cx,", c);
				Code("jle %s", GetLabelR(c));
				break;
			case DIST:
				OpArg1("mov cx,", c);
				OpArg2("cmp cx,", c);
				Code("jne %s", GetLabelR(c));
				break;
			case IGIG:
				OpArg1("mov cx,", c);
				OpArg2("cmp cx,", c);
				Code("je %s", GetLabelR(c));
				break;
			case KW_LABEL:
				fprintf(g_codeOut, "%s:\n", GetLabel1(c));
				break;
			case KW_GOTO:
				Code("jmp %s", GetLabel1(c));
				break;
			case '=':
				if (c.disc & SYM1) {
					Code("mov ax, DS:[%s]", GetName1(c));
					MovResAx(c);
				}
				else
					Code("mov DS:[%s], %dd", GetNameR(c), c.arg1.nConst);
				break;
			case KW_READ:
				bCodeReadInt = true;
				Code("call ReadInt");
				Code("mov DS:[%s], ax", GetNameR(c));
				break;
			case KW_WRITE:
				if (c.disc & STR1) {
					bCodeWriteString = true;
					Code("mov dx, OFFSET str%d", (int) c.arg1.pText);
					Code("call WriteString");
				}
				else {
					bCodeWriteInt = true;
					if (c.disc & SYM1)
						Code("mov ax, DS:[%s]", GetName1(c));
					else
						Code("mov ax, %dd", c.arg1.nConst);

					Code("call WriteInt");
				}
				break;
			case KW_NEWLINE:
				bCodeNewLine = true;
				Code("call NewLine");
				break;
		}
		fputc('\n', g_codeOut);
	}
	
	Code("mov ah, 4ch");	// Fin del programa en DOS
	Code("int 21h");

	// Si es necesario incluir el cdigo de las subfunciones
	
	if (bCodeWriteInt)		CodeWriteInt();
	if (bCodeWriteString)	CodeWriteString();
	if (bCodeReadInt)		CodeReadInt();
	if (bCodeNewLine)		CodeNewLine();

	Code("END");	// Fin del programa en macro assembler
	
	fclose(g_codeOut);
}

void CodeWriteInt()
{
	/* Escribe el entero que este en AX */
	char writeCode[] =
		"\nWriteInt PROC\n"
		"\tmov cx, 0h\n"
		"\tcmp ax, 0d\n"
		"\tjns  NO_SIGN\n"
		"\tpush ax\n"
		"\tmov dl, '-'\n"
		"\tmov ah, 02h\n"
		"\tint 21h\n"
		"\tpop ax\n"
		"\tmov bx, -1d\n"
		"\tmul bx\n"
		"NO_SIGN:\n"
		"\tcmp ax, 10d\n"
		"\tjge BLOQUE_PUSH\n"
		"\tadd al, 48\n"
		"\tmov dl, al\n"
		"\tmov ah, 02h\n"
		"\tint 21h\n"
		"\tjmp FIN_WRITE\n"
		"BLOQUE_PUSH:\n"
		"\tcwd\n"
		"\tmov bx, 10d\n"
		"\tdiv bx\n"
		"\tpush dx\n"
		"\tinc cx\n"
		"\tcmp ax, 0h\n"
		"\tjne BLOQUE_PUSH\n"
		"BLOQUE_PRINT_CARACTER:\n"
		"\tpop ax\n"
		"\tadd al, 48\n"
		"\tmov dl, al\n"
		"\tmov ah, 02h\n"
		"\tint 21h\n"
		"\tdec cx\n"
		"\tcmp cx, 0h\n"
		"\tjne BLOQUE_PRINT_CARACTER\n"
		"FIN_WRITE:\n"
		"\tret\n"
		"WriteInt ENDP\n";

	fprintf(g_codeOut, writeCode);
}

void CodeWriteString()
{
	/* En DX debe estar el puntero al string */
	char writeCode[] = 
		"\nWriteString\tPROC\n"
		"\tmov ah, 9\n"
		"\tint 21h\n"
		"\tret\n"
		"WriteString ENDP\n";
	
	fprintf(g_codeOut, writeCode);
}

void CodeReadInt()
{
	/* El ressultado queda en AX */
	char readCode[] =
		"\nReadInt\tPROC\n"
		"\tmov cx, 0h\n"
		"\tmov bh, 0h\n"
		"LEER_CARACTER: \n"
		"\tmov ah, 01h\n"
		"\tint 21h\n"
		"\tcmp al, 13d\n"
		"\tje fin_read\n"
		"\tcmp al, '0'\n"
		"\tjl LEER_CARACTER_INVALIDO\n"
		"\tcmp al, '9'\n"
		"\tjg LEER_CARACTER_INVALIDO\n"
		"\tmov bl, al\n"
		"\tsub bl, 48\n"
		"\tmov ax, 10d\n"
		"\tmul cx\n"
		"\tmov cx, ax\n"
		"\tadd cx, bx\n"
		"\tjmp LEER_CARACTER\n"
		"LEER_CARACTER_INVALIDO: \n"
		"\tcall FIN_READ\n"
		"\tmov ah, 01h\n"
		"\tint 21h\n"
		"\tcmp al, 0dh\n"
		"\tjne LEER_CARACTER_INVALIDO\n"
		"FIN_READ: \n"
		"\tmov ax, cx\n"
		"\tret\n"
		"ReadInt\tENDP\n";
		
		fprintf(g_codeOut, readCode);
}

void CodeNewLine()
{
	char newLineCode[] =
		"\nNewLine PROC\n"
		"\tpush dx\n"
		"\tpush ax\n"
		"\tmov dl, 10\n"
		"\tmov ah, 02h\n"
		"\tint 21h\n"
		"\tmov dl, 13\n"
		"\tmov ah, 02h\n"
		"\tint 21h\n"
		"\tpop ax\n"
		"\tpop dx\n"
		"\tret\n"
		"NewLine ENDP\n";
		
	fprintf(g_codeOut, newLineCode);
}
