// SharedTypes.h

/* Aca se incluyen los tipos de datos que son
 * compartidos por lex y yyac.
 */
#include "ListTempl.h"
#include <string.h>		// necesaria para el strcmp
#include <stdlib.h>		// necesaria para el atoi

// Constantes del dscriminante de las unions
#define SYM1	2		// Symbol en el argumento 1
#define SYM2	4		// Symbol en el argumento 2
#define SYMR	8
#define CONS1	16		// Constante en el argumento 1
#define CONS2	32		// Constante en el argumento 2
#define LBL1	64		// Label en el argumento 1
#define LBL2	128		// Label en el argumento 2
#define LBLR	256		// Label en el resultado
#define STR1	512		// String en el argumento 1
#define STR2	1024	// String en el argumento 2
#define UNR		2048	// Argumento de resultado no utilizado
#define UN2		4096	// Argumento 2 no utilizado
#define UN1		8192	// Argumento 1 no utilizado
#define JMPR	16384	// Salto no conocido


// Constantes para las caracteristicas en los atributos sitetizados
#define TYPE_CONST	2
#define TYPE_SYMBOL	4
#define TYPE_TEMP	8

// Constantes para los saltos
#define UNKNOWNJUMP 0
#define JUMPTO1     1
#define JUMPTO2		2

struct SYMBOL
{
	char name[255];
	
	bool operator==(const SYMBOL& rhs) const
	{
		return (strcmp(name, rhs.name) == 0);
	}
	void Write(FILE* pf)
	{
		fprintf(pf, "  %s\n", name);
	}
};

struct LABEL
{
	char name[30];
};

typedef NODEPOS PSYMBOL;
typedef NODEPOS PLABEL;
typedef NODEPOS PSTRING;
//typedef unsigned char UCHAR;
 
union ARG {
	PSYMBOL  	pSymbol;
	PLABEL		pLabel;
	PSTRING		pText;
	int		 	nConst;
};
 
struct CUARTETO {
	int 		label;
	int 		disc; // Discrimininante de las dos unions
	ARG 		arg1;
	ARG 		arg2;
	ARG	 		res;
	
	void Write(FILE* pf, CSList<SYMBOL>& symTable, CSList<LABEL>& labels,
		CSList<char*>& stringList)
	{
		if (label < 256)
			fprintf(pf, "(%c, ", label);
		else
			fprintf(pf, "(%s, ", GetLabelName());
		
		if (disc & SYM1)
			fprintf(pf, "%s, ", symTable.GetAt(arg1.pSymbol).name);
		else if (disc & CONS1)
			fprintf(pf, "%d, ", arg1.nConst);
		else if (disc & LBL1)
			fprintf(pf, "%s:, ", labels.GetAt(arg1.pLabel).name);
		else if (disc & STR1)
			fprintf(pf, "\"%s\", ", stringList.GetAt(arg1.pText));
		else if (disc & UN1)
			fprintf(pf, "-, ");
		else
			fprintf(pf, "ERROR EN EL DISCRIMINANTE, ");

		if (disc & SYM2)
			fprintf(pf, "%s, ", symTable.GetAt(arg2.pSymbol).name);
		else if (disc & CONS2)
			fprintf(pf, "%d, ", arg2.nConst);
		else if (disc & LBL2)
			fprintf(pf, "%s:, ", labels.GetAt(arg2.pLabel).name);
		else if (disc & STR2)
			fprintf(pf, "\"%s\", ", stringList.GetAt(arg2.pText));
		else if (disc & UN2)
			fprintf(pf, "-, ");
		else
			fprintf(pf, "ERROR EN EL DISCRIMINANTE, ");

		if (disc & UNR)
			fprintf(pf, "-)\n");
		else if (disc & LBLR)
			fprintf(pf, "%s:)\n", labels.GetAt(res.pLabel).name);
		else if (disc & SYMR)
			fprintf(pf, "%s)\n", symTable.GetAt(res.pSymbol).name);
		else
			fprintf(pf, "ERROR EN EL DISCRIMINANTE)\n");
	}

	char* GetLabelName()
	{
		switch (label) {
			case 257: return "if";
			case 259: return "while";
			case 261: return "Read";
			case 262: return "Write";
			case 263: return "<=";
			case 264: return ">=";
			case 265: return "==";
			case 266: return "!=";
			case 267: return "lbl";
			case 268: return "goto";
			case 269: return "NewLine";
			default:  return "error!!!";
		}
	}
};
 
// Synthesized attribute
struct SA {
	ARG			value;
	int			nType; //Variable, constante, variable temporal u otros
	
	/* Miembros de la lista de cuartetos */
	NODEPOS		m_pLast;
	int			m_nCount;

	CSList<CUARTETO>& GetList() {
		return *((CSList<CUARTETO>*) &m_pLast);
	}
};



