%{
	/* Analizador gramatical, TP2 y TP3
	 * Integrantes: Yanina Manassa, Analia Jahnig, Diego Macrini
	 */           
	 #include <windows.h>
	 #include <stdio.h>     
	 #include "SharedTypes.h"
	 
	 extern "C" FILE *yyout;
	 extern CSList<SYMBOL> symTable;
	 extern CSList<char*>  stringList;
	 extern int g_nLine;
	 extern int g_nErrors;
	 extern bool g_bDeclaracion;
	 
	 double g_fNroVersion = 3.00;
	 
	 CSList<CUARTETO> cuartetos;	// Lista de cuartetos
	 CSList<LABEL>	  labels;
	 CStack<PSYMBOL>  freeVarTemps;	// Pila de variables temporables libres
	 
	 #define ConcatenarCodigo(A, B) A.GetList().AddTail(B.GetList())

	 void AddCuartetoOpBinario(int label, SA& arg1, SA& arg2, SA& res);
	 void AddCuartetoAsign(int label, PSYMBOL arg1, SA& arg2, SA& res);
	 void AddCuartetoMenosUnario(SA& arg1, SA& res);
	 void AddCuartetoWhile(SA& condicion, SA& sentencia, SA& res);
	 void AddCuartetoIf(SA& condicion, SA& sentencia, SA& res);
	 void AddCuartetoIfElse(SA& condicion, SA& sentencia, SA& sentElse, SA& res);	 
	 void AddCuartetoRead(int label, PSYMBOL arg1, SA& res);
	 void AddCuartetoWrite(int label, PSTRING arg1, SA& res);
	 void AddCuartetoWrite(int label, SA& arg1, SA& res);
	 void AddCuartetoNewLine(int label, SA& res);
	 void AddCuartetoComp(int label, SA& arg1, SA& arg2, SA& res);
	 void AddCuartetoOR(int label, SA& arg1, SA& arg2, SA& res);
	 void AddCuartetoAND(int label, SA& arg1, SA& arg2, SA& res);
	 
	 extern void GenerarCodigo(char*);
	 
	 void ErrorMessage(char* szMsg);
%}

%union {
	SA 	 		sa;
	PSYMBOL 	pSymbol;
	PSTRING		pText;
	int			nConst;
	int 		nCompCode;

	tagYYSTYPE()	// Constructor de la union
	{
		/* Idem constructor de la lista */
		sa.m_pLast = NULL;
		sa.m_nCount = 0;
		
		//ErrorMessage("Se creo la union");
	}
}       
       
%token KW_IF KW_ELSE KW_WHILE KW_INT KW_READ KW_WRITE
%token MENIG MAYIG IGIG DIST KW_LABEL KW_GOTO KW_NEWLINE

%token <pSymbol>   	ID
%token <nConst> 	CONS 
%token <nCompCode> 	COMP
%token <pText>		STRING

%type <sa> programa sentencia lista_sentencias
%type <sa> s_if s_while
%type <sa> expresion condicion

%left '-' '+'
%left '*' '/'
%left '|' '&'
%nonassoc MENOS_UNARIO

%nonassoc LOWER_THAN_ELSE
%nonassoc KW_ELSE

%%
programa:   sentencia { cuartetos.AddTail($1.GetList()); }
	|		programa sentencia { cuartetos.AddTail($2.GetList()); }
	;

sentencia: 	ID '=' expresion ';'	{
				AddCuartetoAsign('=', $1, $3, $$);
				fprintf(yyout, "Sentencia de asignacin valida\n");
			}
	|		expresion ';' {
				ConcatenarCodigo($$, $1);
				fprintf(yyout, "Sentencia valida\n");
			}
	|		s_if {
				ConcatenarCodigo($$, $1);
				fprintf(yyout, "Sentencia if valida\n");
			}
	|		s_while	{
				ConcatenarCodigo($$, $1);
				fprintf(yyout, "Sentencia while valida\n");
			}
	| 		'{' lista_sentencias '}'  {
				ConcatenarCodigo($$, $2);
				fprintf(yyout, "Sentencia compuesta valida\n");
			}
	|		ID '=' KW_READ '(' ')' ';' {
				AddCuartetoRead(KW_READ, $1, $$);
				fprintf(yyout, "Sentencia Read valida\n");
			}
	|		KW_WRITE '(' expresion ')' ';' {
				AddCuartetoWrite(KW_WRITE, $3, $$);
				fprintf(yyout, "Sentencia Write expresin valida\n");
			}
	|		KW_WRITE '(' STRING ')' ';' {
				AddCuartetoWrite(KW_WRITE, $3, $$);
				fprintf(yyout, "Sentencia Write string valida\n");
			}
	|		KW_NEWLINE '(' ')' ';'{
				AddCuartetoNewLine(KW_NEWLINE, $$);
				fprintf(yyout, "Sentencia NewLine valida\n");
			}
	|		declaracion ';'	{ 
				// No genera cdigo
				g_bDeclaracion = false;
				fprintf(yyout, "Delaracin valida\n"); 
			} 
	;
	
lista_sentencias: sentencia { 
				ConcatenarCodigo($$, $1); 
			}
	|		lista_sentencias sentencia { 
				ConcatenarCodigo($$, $1);
				ConcatenarCodigo($$, $2);
			}
	;
	
declaracion: KW_INT ID
	|		 declaracion ',' ID
	;

s_if:		KW_IF '(' condicion ')' sentencia 	%prec LOWER_THAN_ELSE {
				AddCuartetoIf($3, $5, $$);	
			}
	|		KW_IF '(' condicion ')' sentencia KW_ELSE sentencia {
				AddCuartetoIfElse($3, $5, $7, $$);
			}
	;                                                                 

s_while:	KW_WHILE '(' condicion ')' sentencia {
				AddCuartetoWhile($3, $5, $$);
			}                                                                
	;
	
condicion:	expresion COMP expresion { 
				AddCuartetoComp($2, $1, $3, $$);
			}
	|		condicion '|' condicion { 
				AddCuartetoOR('|',$1,$3,$$); 
			}
	|		condicion '&' condicion { 
				AddCuartetoAND('&',$1,$3,$$); 
			}
	;

expresion:	expresion '+' expresion { AddCuartetoOpBinario('+',$1,$3,$$); }
	|		expresion '-' expresion { AddCuartetoOpBinario('-',$1,$3,$$); }
	|		expresion '*' expresion { AddCuartetoOpBinario('*',$1,$3,$$); }
	|		expresion '/' expresion { AddCuartetoOpBinario('/',$1,$3,$$); }
	
	|		'-' expresion %prec MENOS_UNARIO { AddCuartetoMenosUnario($2, $$); }
	|		'(' expresion ')' { $$ = $2; }  // Ojo aca
	
	|		ID { 
				$$.value.pSymbol = $1; 
				$$.nType = TYPE_SYMBOL;
			}                             
			
	|		CONS { 
				$$.value.nConst = $1; 
				$$.nType = TYPE_CONST;
			} 
	; 

%%
extern "C" FILE *yyin;

int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance,
				   PSTR szCmdLine, int iCmdShow)
{
	int ret = 0;
	if (__argc != 4) 
	{
		ErrorMessage("Error en la lnea de comando");
		return 1;
	}
	if( (yyin  = fopen(__argv[1], "rt" )) == NULL ) 
	{
		ErrorMessage("Error al abrir el archivo de entrada");
		return 1;
	}
	if((yyout = fopen(__argv[2], "wt" )) == NULL ) 
	{
		fclose(yyin);
		ErrorMessage("Error al abrir el archivo de salida");
		return 1;
	}
    
    fprintf(yyout, "Inicio del parsing ( versin %.2f )\n\n", g_fNroVersion);
    
	while(!feof(yyin)) {
		yyparse();
	}
	
	// Fin del parsing. Si no hubo errores comenzar la
	// generacin de cdigo.
	if (g_nErrors == 0) {
		fprintf(yyout, "\nIniciando generacin de cdigo...\n");
		ret = 0;
		GenerarCodigo(__argv[3]);
	}
	else {
		fprintf(yyout, "\nGeneracin de cdigo cancelada\n");
		ret = 500;	// Indica que se cancelo la generacin de cdigo
	}
		
	/* Guardar en el archivo de salida el estado actual */
	fprintf(yyout, "\nTabla de smbolos\n\n");
	NODEPOS pos = symTable.GetHeadPos();
	
	while (pos)
		symTable.GetNext(pos).Write(yyout);
		
	fprintf(yyout, "\nCuartetos\n\n");	
	
	pos = cuartetos.GetHeadPos();
	
	while (pos)
		cuartetos.GetNext(pos).Write(yyout, symTable, labels, stringList);
		
	// La lista de strings es de punteros y hay que eliminar su contenido.
	pos = stringList.GetHeadPos();
	
	while (pos)
		delete[] stringList.GetNext(pos);
		
	fprintf(yyout, "\n-------------------------------------------------------------\n");
	fprintf(yyout, "%s -- errores %d, advertencias 0\n", __argv[1], g_nErrors);

	 _fcloseall( );

	return ret;
}

void yyerror(const char* s)
{
	int c;
	
	g_nErrors++;
	
	if (feof(yyin))
		fprintf(yyout, "ERROR: inesperado fin de archivo\n");
	else
		fprintf(yyout, "ERROR: sintxis desconocida en lnea: %d\n", g_nLine);
	
	do {
		c = getc(yyin);
		if (c == '\n') g_nLine++;
	} while (c != ';' && c != EOF);
}

void ErrorMessage(char* szMsg)
{
	MessageBox(NULL, szMsg, "Error", MB_OK | MB_ICONERROR);
}

///////////////////////////////////////////////////////////////////////
// Funciones auxiliares para la generacin de cuartetos

PSYMBOL GetVarTemp()
{
	static i = 0;
	
	if (freeVarTemps.IsEmpty()) {
		SYMBOL sym;
		
		strcpy(sym.name, "_Tmp");
		_itoa(i++, sym.name + 4, 10);
		
		symTable.AddTail(sym);
		
		return symTable.GetTailPos();
	}
	else
		return freeVarTemps.Pop();
}

PLABEL GetLabel()
{
	static i = 0;
	
	LABEL label;
	
	strcpy(label.name, "LABEL");
	_itoa(i++, label.name + 5, 10);
	labels.AddTail(label);
	
	return labels.GetTailPos();
}

void FreeVarTemp(PSYMBOL pSymbol)
{
	freeVarTemps.Push(pSymbol);
}

void InvertComp(int& comp)
{
	switch (comp) {
		case '>': comp = MENIG; break;
		case '<': comp = MAYIG; break;
		case MENIG: comp = '>'; break;
		case MAYIG: comp = '<'; break;
		case IGIG: comp = DIST; break;
		case DIST: comp = IGIG; break;
	}
}

///////////////////////////////////////////////////////////////////////
// Funciones generadoras de cuartetos

void AddCuartetoOpBinario(int label, SA& arg1, SA& arg2, SA& res)
{	
	CUARTETO c;
	
	ConcatenarCodigo(res, arg1);
	ConcatenarCodigo(res, arg2);
	
	// Si arg1 o arg2 son variables temporales, las liberamos
	// para que puedan se reutilizadas.
	if (arg1.nType & TYPE_TEMP)
		FreeVarTemp(arg1.value.pSymbol);		
	if (arg2.nType & TYPE_TEMP)
		FreeVarTemp(arg2.value.pSymbol);

	// Obtenemos una variable temporal para almacenar el resultado
	res.value.pSymbol = GetVarTemp();
	res.nType = TYPE_SYMBOL | TYPE_TEMP;

	// Aadimos el cuarteto de la operacin
	c.label = label;
	c.arg1 = arg1.value;
	c.arg2 = arg2.value;
	c.res = res.value;
	c.disc = SYMR;
	c.disc |= (arg1.nType & TYPE_SYMBOL) ? SYM1:CONS1;
	c.disc |= (arg2.nType & TYPE_SYMBOL) ? SYM2:CONS2;	
	res.GetList().AddTail(c);
}

void AddCuartetoAsign(int label, PSYMBOL arg1, SA& arg2, SA& res)
{
	CUARTETO c;
	
	ConcatenarCodigo(res, arg2);
	
	// Si arg2 es una variable temporal, la liberamos.
	if (arg2.nType & TYPE_TEMP)
		FreeVarTemp(arg2.value.pSymbol);
	
	// El resultado debe quedar en arg1	                                  
	res.value.pSymbol = arg1;
	res.nType = TYPE_SYMBOL;

	// Advertencia: arg2 debe ser el arg1 del cuarteto
	// y el arg2 del cuarteto no se utiliza.
	c.label = label;
	c.arg1 = arg2.value;
	c.res = res.value;
	c.disc = SYMR;
	c.disc |= (arg2.nType & TYPE_SYMBOL) ? SYM1:CONS1;
	c.disc |= UN2;	// arg2 no utilizado
	res.GetList().AddTail(c);
}

void AddCuartetoMenosUnario(SA& arg1, SA& res)
{
	if (arg1.nType & TYPE_SYMBOL) {
		SA arg2;
		arg2.value.nConst = -1; 
		arg2.nType = TYPE_CONST;
		arg2.m_pLast = NULL;
		arg2.m_nCount = 0;
		AddCuartetoOpBinario('*', arg1, arg2, res);
	}
	else if (arg1.nType & TYPE_CONST) {
		res.value.nConst = arg1.value.nConst * -1; 
		res.nType = TYPE_CONST;
	}
	else
		ErrorMessage("DEBUG");
}

void AddCuartetoWhile(SA& condicion, SA& sentencia, SA& res)
{
	CUARTETO c;
	PLABEL labelBegin, labelCode, labelEnd; 
	bool bAddLabelCode = false;
	
	labelBegin = GetLabel();
	labelEnd = GetLabel();
	
	// Agregar etiqueta origen
	c.label = KW_LABEL;
	c.arg1.pLabel = labelBegin;
	c.disc = LBL1 | UN2 | UNR;	// Label en 1 y agr2 y res no usados
	res.GetList().AddTail(c);
	
	NODEPOS pos = condicion.GetList().GetHeadPos();
	
	while (pos) {
		CUARTETO& cc = condicion.GetList().GetNext(pos);
		
		if (cc.disc & JMPR) {
			switch (cc.res.nConst) {
				case JUMPTO1:
					if (!bAddLabelCode) {
						bAddLabelCode = true;
						labelCode = GetLabel();
					}
					cc.res.pLabel = labelCode; 
					break;
			 	case JUMPTO2:
			 		cc.res.pLabel = labelEnd; 
			 		break;
			 	case UNKNOWNJUMP:
			 		InvertComp(cc.label);
			 		cc.res.pLabel = labelEnd; 
			 		break;
			 	default:
			 		ErrorMessage("DEBUG");
			 }
			 cc.disc &= ~JMPR;
			 cc.disc |= LBLR;
		}
	}
	
	// Agregar el cdigo de la condicin
	ConcatenarCodigo(res, condicion);
	                         
	if (bAddLabelCode) {
		// Agregar etiqueta comienzo
		c.label = KW_LABEL;
		c.arg1.pLabel = labelCode;
		c.disc = LBL1 | UN2 | UNR;
		res.GetList().AddTail(c);
	}
	
	// Agregar el cdigo de la/s sentencia/s	                         
	ConcatenarCodigo(res, sentencia);
	
	// Agregar salto incondicional al comienzo
	c.label = KW_GOTO;
	c.arg1.pLabel = labelBegin;
	c.disc = LBL1 | UN2 | UNR;
	res.GetList().AddTail(c);
	
	// Agregar etiqueta final
	c.label = KW_LABEL;
	c.arg1.pLabel = labelEnd;
	c.disc = LBL1 | UN2 | UNR;
	res.GetList().AddTail(c);
}      

void AddCuartetoIf(SA& condicion, SA& sentencia, SA& res)
{
	CUARTETO c;
	PLABEL labelBegin, labelEnd;
	bool bAddLabelBegin = false;

	labelEnd = GetLabel();
	
	NODEPOS pos = condicion.GetList().GetHeadPos();
	
	while (pos) {
		CUARTETO& cc = condicion.GetList().GetNext(pos);
		
		if (cc.disc & JMPR) {
			switch (cc.res.nConst) {
				case JUMPTO1:
					if (!bAddLabelBegin) {
						bAddLabelBegin = true;
						labelBegin = GetLabel();
					}
					cc.res.pLabel = labelBegin; 
					break;
			 	case JUMPTO2:
			 		cc.res.pLabel = labelEnd; 
			 		break;
			 	case UNKNOWNJUMP:
			 		InvertComp(cc.label);
			 		cc.res.pLabel = labelEnd; 
			 		break;
			 	default:
			 		ErrorMessage("DEBUG");
			 }
			 cc.disc &= ~JMPR;
			 cc.disc |= LBLR;
		}
	}
	
	// Agregar el cdigo de la condicin
	ConcatenarCodigo(res, condicion); 
	
	if (bAddLabelBegin) {
		// Agregar etiqueta comienzo
		c.label = KW_LABEL;
		c.arg1.pLabel = labelBegin;
		c.disc = LBL1 | UN2 | UNR;
		res.GetList().AddTail(c);
	}
	
	// Agregar el cdigo de la/s sentencia/s	                         
	ConcatenarCodigo(res, sentencia);
	
	// Agregar etiqueta final
	c.label = KW_LABEL;
	c.arg1.pLabel = labelEnd;
	c.disc = LBL1 | UN2 | UNR;
	res.GetList().AddTail(c);
}

void AddCuartetoIfElse(SA& condicion, SA& sentencia, SA& sentElse, SA& res)
{
	CUARTETO c;
	PLABEL labelBegin, labelElse, labelEnd;
	bool bAddLabelBegin = false;

	labelElse = GetLabel();
	labelEnd = GetLabel();
	
	NODEPOS pos = condicion.GetList().GetHeadPos();
	
	while (pos) {
		CUARTETO& cc = condicion.GetList().GetNext(pos);
		
		if (cc.disc & JMPR) {
			switch (cc.res.nConst) {
				case JUMPTO1:
					if (!bAddLabelBegin) {
						bAddLabelBegin = true;
						labelBegin = GetLabel();
					}
					cc.res.pLabel = labelBegin; 
					break;
			 	case JUMPTO2:
			 		cc.res.pLabel = labelElse; 
			 		break;
			 	case UNKNOWNJUMP:
			 		InvertComp(cc.label);
			 		cc.res.pLabel = labelElse; 
			 		break;
			 	default:
			 		ErrorMessage("DEBUG");
			 }
			 cc.disc &= ~JMPR;
			 cc.disc |= LBLR;
		}
	}
	
	// Agregar el cdigo de la condicin
	ConcatenarCodigo(res, condicion); 
	
	if (bAddLabelBegin) {
		// Agregar etiqueta comienzo
		c.label = KW_LABEL;
		c.arg1.pLabel = labelBegin;
		c.disc = LBL1 | UN2 | UNR;
		res.GetList().AddTail(c);
	}
	
	// Agregar el cdigo de la/s sentencia/s	                         
	ConcatenarCodigo(res, sentencia);
	
	// Agregar salto incondicional al final
	c.label = KW_GOTO;
	c.arg1.pLabel = labelEnd;
	c.disc = LBL1 | UN2 | UNR;
	res.GetList().AddTail(c);
	
	// Agregar etiqueta del else
	c.label = KW_LABEL;
	c.arg1.pLabel = labelElse;
	c.disc = LBL1 | UN2 | UNR;
	res.GetList().AddTail(c);
	
	// Agregar el cdigo de la/s sentencia/s del else
	ConcatenarCodigo(res, sentElse);
	
	// Agregar etiqueta final
	c.label = KW_LABEL;
	c.arg1.pLabel = labelEnd;
	c.disc = LBL1 | UN2 | UNR;
	res.GetList().AddTail(c);
}

void AddCuartetoRead(int label, PSYMBOL arg1, SA& res)
{
	CUARTETO c;

	c.label = label;
	c.res.pSymbol = arg1;
	c.disc = UN1 | UN2 | SYMR;
	res.GetList().AddTail(c);
}

void AddCuartetoWrite(int label, PSTRING arg1, SA& res)
{
	CUARTETO c;
	
	c.label = label;
	c.arg1.pText = arg1;
	c.disc = STR1 | UN2 | UNR;
	
	res.GetList().AddTail(c);
} 

void AddCuartetoNewLine(int label, SA& res)
{
	CUARTETO c;
	
	c.label = label;
	c.disc = UN1 | UN2 | UNR;
	
	res.GetList().AddTail(c);
}

void AddCuartetoWrite(int label, SA& arg1, SA& res)
{
	CUARTETO c;
	
	// Agregar el cdigo de la expresin en arg1
	ConcatenarCodigo(res, arg1);
	
	// Aadir el cuarteto de la operacin
	c.label = label;
	c.arg1 = arg1.value;
	c.disc = (arg1.nType & TYPE_SYMBOL) ? SYM1:CONS1;
	c.disc |= UN2 | UNR;	
	res.GetList().AddTail(c);
	
	// Si la arg1 es una variable temporal, la liberamos.
	if (arg1.nType & TYPE_TEMP)
		FreeVarTemp(arg1.value.pSymbol);
}

void AddCuartetoComp(int label, SA& arg1, SA& arg2, SA& res)
{
	CUARTETO c;
	
	ConcatenarCodigo(res, arg1);
	ConcatenarCodigo(res, arg2);
	
	// Si arg1 o arg2 son variables temporales, las liberamos
	// para que puedan se reutilizadas.
	if (arg1.nType & TYPE_TEMP)
		FreeVarTemp(arg1.value.pSymbol);		
	if (arg2.nType & TYPE_TEMP)
		FreeVarTemp(arg2.value.pSymbol);

	// Aadimos el cuarteto de la operacin
	c.label = label;
	c.arg1 = arg1.value;
	c.arg2 = arg2.value;
	c.res.nConst = UNKNOWNJUMP;	// Todavia no se donde hay que saltar
	
	c.disc = JMPR;	// El resultado no contiene nada relevante
	c.disc |= (arg1.nType & TYPE_SYMBOL) ? SYM1:CONS1;
	c.disc |= (arg2.nType & TYPE_SYMBOL) ? SYM2:CONS2;
	
	res.GetList().AddTail(c);
}

void AddCuartetoOR(int label, SA& arg1, SA& arg2, SA& res)
{
	CSList<CUARTETO>& listArg1 = arg1.GetList();
	
	if (listArg1.GetAt(listArg1.GetTailPos()).res.nConst != UNKNOWNJUMP) 
	{
		PLABEL labelJump;
		
		labelJump = GetLabel();
		
		NODEPOS pos = listArg1.GetHeadPos();
		
		while (pos) {
			CUARTETO& c = listArg1.GetNext(pos);
			
			if ((c.disc & JMPR) && c.res.nConst == JUMPTO2) {
				if (pos) {	// => no es el ltimo
					c.res.pLabel = labelJump;
					c.disc &= ~JMPR;	// Ahora si conoce el salto
					c.disc |= LBLR;
				}
				else {		// Es el ltimo
					InvertComp(c.label);
					c.res.nConst = JUMPTO1;
				}	
			}
		}
		
		ConcatenarCodigo(res, arg1);
		
		CUARTETO c;
		// Agregar etiqueta del jump
		c.label = KW_LABEL;
		c.arg1.pLabel = labelJump;
		c.disc = LBL1 | UN2 | UNR;
		res.GetList().AddTail(c);
	}
	else {
		// El ltimo seguro es un COMP
		CUARTETO& c = listArg1.GetAt(listArg1.GetTailPos());
		
		c.res.nConst = JUMPTO1;
		
		ConcatenarCodigo(res, arg1);
	}
	
	NODEPOS pos = arg2.GetList().GetTailPos();	// El ltimo seguro es un COMP
	CUARTETO& c = arg2.GetList().GetAt(pos);
	
	InvertComp(c.label);
	
	c.res.nConst = JUMPTO2;
	
	ConcatenarCodigo(res, arg2);
}

void AddCuartetoAND(int label, SA& arg1, SA& arg2, SA& res)
{
	CSList<CUARTETO>& listArg1 = arg1.GetList();
	
	if (listArg1.GetAt(listArg1.GetTailPos()).res.nConst != UNKNOWNJUMP) 
	{
		PLABEL labelJump;
		
		labelJump = GetLabel();
		
		NODEPOS pos = listArg1.GetHeadPos();
		
		while (pos) {
			CUARTETO& c = listArg1.GetNext(pos);
			
			if ((c.disc & JMPR) && c.res.nConst == JUMPTO1) {
				c.res.pLabel = labelJump;
				c.disc &= ~JMPR;	// Ahora si conoce el salto
				c.disc |= LBLR;
			}
		}  
		
		ConcatenarCodigo(res, arg1);
		
		CUARTETO c;
		// Agregar etiqueta del jump
		c.label = KW_LABEL;
		c.arg1.pLabel = labelJump;
		c.disc = LBL1 | UN2 | UNR;
		res.GetList().AddTail(c);
	}
	else {
		// El ltimo seguro es un COMP
		CUARTETO& c = arg1.GetList().GetAt(listArg1.GetTailPos());
		
		InvertComp(c.label);
		c.res.nConst = JUMPTO2;
		
		ConcatenarCodigo(res, arg1);
	}
	
	NODEPOS pos = arg2.GetList().GetTailPos();	// El ltimo seguro es un COMP
	CUARTETO& c = arg2.GetList().GetAt(pos);
	
	InvertComp(c.label);
	
	c.res.nConst = JUMPTO2;
	
	ConcatenarCodigo(res, arg2);
}



	
