%{
	/* Anlaizador lexico TP1
	 * Integrantes: Yanina Manassa, Analia Jahnig, Diego Macrini
	*/                    
	extern void ErrorMessage(char* szMsg);
	#include "SharedTypes.h"
	#include "gramatica.h"
	
	extern "C" FILE *yyout;
	
	CSList<SYMBOL> symTable;
	CSList<char*>  stringList;
	int g_nLine = 0;
	int g_nErrors = 0;
	bool g_bDeclaracion = false;
	
	enum ERRORID {STRINGERROR, DBLDECL, UNKNOWNID};
	void OnError(ERRORID errorID);	
%}

string \"[^"\n]*["\n]
comentario "//".*\n

%%
[\t ]+		 ; /* Ignora espacios en blanco */
{comentario} g_nLine++; /* Ignora comentarios */

if 			return KW_IF;
else 		return KW_ELSE;
while 		return KW_WHILE;
int			{ g_bDeclaracion = true; return KW_INT; }

Read		return KW_READ;
Write		return KW_WRITE;
NewLine		return KW_NEWLINE;

">"  		{ yylval.nCompCode = '>';   return COMP; }
"<"  		{ yylval.nCompCode = '<';   return COMP; }
"<=" 		{ yylval.nCompCode = MENIG; return COMP; }
">=" 		{ yylval.nCompCode = MAYIG; return COMP; }
"==" 		{ yylval.nCompCode = IGIG;  return COMP; }
"!=" 		{ yylval.nCompCode = DIST;  return COMP; }

{string} 	{
				if (yytext[yyleng - 1] != '"') OnError(STRINGERROR);
				else if (yytext[yyleng - 2] == '\\') {   // Caso de barra comillas
					yyless(yyleng - 1);
					yymore();
				}
				else {
					char* p = new char[yyleng - 2 + 1];	// Menos 2 comillas ms un \0
					memcpy(p, yytext + 1, yyleng - 2);	// Descarta las comillas
					p[yyleng - 2] = '\0';				// Se asigna nulo al final
					stringList.AddTail(p);
					yylval.pText = stringList.GetTailPos();
					return STRING;
				}
			}

[a-zA-Z]+ 	{				
				SYMBOL sym;
				strcpy(sym.name, yytext);
				NODEPOS pos = symTable.Find(sym);
				if (pos) {		// Encontro el identificador
					yylval.pSymbol = pos;
					if (g_bDeclaracion) OnError(DBLDECL);
				}
				else {	// No lo encontro
					if (!g_bDeclaracion) OnError(UNKNOWNID);
					// Lo aade en ambos casos para seguir compilando
					symTable.AddTail(sym);
					yylval.pSymbol = symTable.GetTailPos();
				}
					
				return ID;
			}

[0-9]+ 		{ yylval.nConst = atoi(yytext); return CONS; }

\n			g_nLine++;
.			return yytext[0];

%%
void OnError(ERRORID errorID)
{
	g_nErrors++;
	fprintf(yyout, "ERROR: ");
	
	switch (errorID) {
		case STRINGERROR:
			fprintf(yyout, "falta cerrar comillas");
			break;
		case DBLDECL:
			fprintf(yyout, "doble declaracin de %s", yytext);
			break;
		case UNKNOWNID:
			fprintf(yyout, "identificador %s no declarado", yytext);
			break;
		default:
			fprintf(yyout, "desconocido");
		    break;			
	}
	
	fprintf(yyout, " en lnea %d\n", g_nLine);
}