/*------------------------------------------------
 PASADA1.CPP
--------------------------------------------------*/
#include <windows.h>
//#include <afxwin.h>
//#include <afxtempl.h>
#include <stdio.h>
#include "Constantes.h"
#include "matriz.h"

struct TOKEN {
	int tokenId;
	int tokenIndex;
	int lineNumber;
	int errorId;
	char error[255];
};

/* Prototipos */
int GetToken(TOKEN* pToken);
int GetEvent(int c);
void ErrorMessage(char* szMsg);
void PrintOutput(int token);

/* Variables globales */
FILE *in, *out;
char lexBuffer[MAXLEXBUFFER], lexBufferLength = 0;
int nLineCount = 0, nLineTokenCount = 0;
bool bAddNewLine = false;


int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance,
				   PSTR szCmdLine, int iCmdShow)
{
	if (__argc != 3) 
	{
		ErrorMessage("Error en la lnea de comando");
		return 1;
	}
	if( (in  = fopen(__argv[1], "rt" )) == NULL ) 
	{
		ErrorMessage("Error al abrir el archivo de entrada");
		return 1;
	}
	if((out = fopen(__argv[2], "wt" )) == NULL ) 
	{
		fclose(in);
		ErrorMessage("Error al abrir el archivo de salida");
		return 1;
	}

	TOKEN token;

	while (GetToken(&token) != EOF) 
	{
		PrintOutput(token.tokenId);
	}

	 _fcloseall( );

	return 0;
}

int GetEvent(int c)
{
	//if (c == EOF) return EOF;
	if (isalpha(c) || c == '_') return 0;
	if (isdigit(c)) return 1;
	if (c == '/') return 2;
	if (c == '*') return 3;
	if (c == '<' || c == '>' || c == '=' || c == '!') 
		return 4;
	if (c == '+' || c == '-' || c == '*' || c == '%') 
		return 5;
	if (c == '\t' || c == ' ' || c == '\v' 
		|| c == '\f' || c == EOF) 
		return 6;
	if (c == '\n') {
		bAddNewLine = true;
		nLineCount++;
		return 6;
	}
	if (c == '"') return 7;
	if (c == '\\') return 8;
	if (c == ';') 
	{
		nLineTokenCount = 0;
		return 9;
	}

	if (c == '(' || c == ')' || 
		c == '[' || c == ']' || 
		c == '{' || c == '}' || 
		c == ':') return 9;

	return EOF;
}

int GetToken(TOKEN* pToken)
{
	int c, event, state = 0, token;
	int (*pf)(char);

	lexBufferLength = 0;

	do {
		c = fgetc(in);
		event = GetEvent(c);
		//if (event == EOF) return EOF;

		if (pf = pfSemRut[state][event]) 
			token = (*pf)((char)c);

		state = newState[state][event];

	} while (state != END);

	nLineTokenCount++;

	pToken->tokenId = token;

	return 0;
}

void ErrorMessage(char* szMsg)
{
	MessageBox(NULL, szMsg, "Error", MB_OK | MB_ICONERROR);
}

int OnID(char c)
{
	if (lexBufferLength < 11)
 		lexBuffer[lexBufferLength++] = c;
	
	return TK_ID;
}

int OnCons(char c)
{
	if (lexBufferLength < 11)
		lexBuffer[lexBufferLength++] = c;

	return TK_CONS;
}

int OnOpCmp(char c)
{
	int nc = fgetc(in);
	if (nc != '=') ungetc(nc, in);

	return TK_COMP;
}

int OnOpArit(char c)
{
	int nc = fgetc(in);
	if (nc != '=') ungetc(nc, in);

	return TK_ARIT;
}

int OnEndID(char c)
{
	ungetc(c, in);
	lexBuffer[lexBufferLength] = '\0';
	int token;

	/*if (nLineTokenCount == 0) {
		int nc = fgetc(in);
		if (nc == ':') return TK_ID_LABEL;
		else ungetc(nc, in);
	}*/

	if (!strcmp(lexBuffer, "if")) token = TK_KEYWORD_IF;
	else if (!strcmp(lexBuffer, "else")) token = TK_KEYWORD_ELSE;
	else if (!strcmp(lexBuffer, "return")) token = TK_KEYWORD_RETURN;
	else if (!strcmp(lexBuffer, "do")) token = TK_KEYWORD_DO;
	else if (!strcmp(lexBuffer, "while")) token = TK_KEYWORD_WHILE;
	else if (!strcmp(lexBuffer, "for")) token = TK_KEYWORD_FOR;
	else if (!strcmp(lexBuffer, "switch")) token = TK_KEYWORD_SWITCH;
	else if (!strcmp(lexBuffer, "case")) token = TK_KEYWORD_CASE;
	else token = TK_ID_VARIABLE;

	return token;
}

int OnEndCons(char c)
{
	ungetc(c, in);
	
	return TK_CONS_INTEGER;
}

int OnStr(char c)
{
	static int comillas = 0;

	if (c == '\\') {
		int nc = fgetc(in);
		if (nc == '"') 
			lexBuffer[lexBufferLength++] = nc;
		else {
			ungetc(nc, in);
			lexBuffer[lexBufferLength++] = c;
		}
	}

	return TK_CONS_STRING;
}

int OnEndStr(char c)
{
	return TK_CONS_STRING;
}

int OnPunt(char c)
{
	switch (c) 
	{
		case '(': return TK_PUNT_PA;
		case ')': return TK_PUNT_PC;
		case '{': return TK_PUNT_LLA;
		case '}': return TK_PUNT_LLC;
		case '[': return TK_PUNT_CA;
		case ']': return TK_PUNT_CC;
		case ':': return TK_PUNT_DOSP;
	}
	return TK_ERROR;
}

int OnError(char c)
{
	return TK_ERROR;
}

void PrintOutput(int token)
{
	switch (token) 
	{
		case TK_ID_VARIABLE: 
			fprintf(out, "variable(%d), ", token); 
			break;
		case TK_CONS_INTEGER: 
			fprintf(out, "constante(%d), ", token); 
			break;
		case TK_OPERATOR_DIV: 
			fprintf(out, "div(%d), ", token); 
			break;
		case TK_COMP: 
			fprintf(out, "comp(%d), ", token); 
			break;
		case TK_ARIT: 
			fprintf(out, "arit(%d), ", token); 
			break;
		case TK_KEYWORD_IF: 
			fprintf(out, "if(%d), ", token); 
			break;
		case TK_KEYWORD_ELSE: 
			fprintf(out, "else(%d), ", token); 
			break;
		case TK_KEYWORD_RETURN: 
			fprintf(out, "return(%d), ", token); 
			break;
		case TK_KEYWORD_DO: 
			fprintf(out, "do(%d), ", token); 
			break;
		case TK_KEYWORD_WHILE: 
			fprintf(out, "while(%d), ", token); 
			break;
		case TK_KEYWORD_FOR: 
			fprintf(out, "for(%d), ", token); 
			break;
		case TK_KEYWORD_SWITCH: 
			fprintf(out, "switch(%d), ", token); 
			break;
		case TK_KEYWORD_CASE: 
			fprintf(out, "case(%d), ", token); 
			break;
		case TK_CONS_STRING: 
			fprintf(out, "string(%d), ", token); 
			break;
		case TK_ID_LABEL: 
			fprintf(out, "label(%d), ", token); 
			break;
		case TK_PUNT_PA: 
		case TK_PUNT_PC: 
		case TK_PUNT_LLA:
		case TK_PUNT_LLC:
		case TK_PUNT_CA:
		case TK_PUNT_CC:
		case TK_PUNT_DOSP: 
			fprintf(out, "punt(%d), ", token); 
			break;
		case TK_ERROR:
			fprintf(out, "error(%d), ", token); 
			break;
		default: 
			fprintf(out, "raro(%d), ", token); 
			break;
	}
	if (bAddNewLine) {
		fputc('\n', out);
		bAddNewLine = false;
	}

}